@@ -38,7 +38,7 @@
 #pragma once
 
 #include "Exception.h"
-#include "pathmatcher/IPathMatcher.h"
+#include "Filter.h"
 
 #include <system_error>
 
@@ -115,9 +115,9 @@ class copy : public QObject {
         m_followSymlinks = follow;
         return *this;
     }
-    copy& matcher(const IPathMatcher* filter)
+    copy& matcher(Filter filter)
     {
-        m_matcher = filter;
+        m_matcher = std::move(filter);
         return *this;
     }
     copy& whitelist(bool whitelist)
@@ -147,7 +147,7 @@ class copy : public QObject {
 
    private:
     bool m_followSymlinks = true;
-    const IPathMatcher* m_matcher = nullptr;
+    Filter m_matcher = nullptr;
     bool m_whitelist = false;
     bool m_overwrite = false;
     QDir m_src;
@@ -209,9 +209,9 @@ class create_link : public QObject {
         m_useHardLinks = useHard;
         return *this;
     }
-    create_link& matcher(const IPathMatcher* filter)
+    create_link& matcher(Filter filter)
     {
-        m_matcher = filter;
+        m_matcher = std::move(filter);
         return *this;
     }
     create_link& whitelist(bool whitelist)
@@ -260,7 +260,7 @@ class create_link : public QObject {
 
    private:
     bool m_useHardLinks = false;
-    const IPathMatcher* m_matcher = nullptr;
+    Filter m_matcher = nullptr;
     bool m_whitelist = false;
     bool m_recursive = true;
 
@@ -291,6 +291,8 @@ bool move(const QString& source, const QString& dest);
  */
 bool deletePath(QString path);
 
+bool removeFiles(QStringList listFile);
+
 /**
  * Trash a folder / file
  */
@@ -353,14 +355,18 @@ bool checkProblemticPathJava(QDir folder);
 // Get the Directory representing the User's Desktop
 QString getDesktopDir();
 
+// Get the Directory representing the User's Applications directory
+QString getApplicationsDir();
+
 // Overrides one folder with the contents of another, preserving items exclusive to the first folder
 // Equivalent to doing QDir::rename, but allowing for overrides
 bool overrideFolder(QString overwritten_path, QString override_path);
 
 /**
  * Creates a shortcut to the specified target file at the specified destination path.
+ * Returns null QString if creation failed; otherwise returns the path to the created shortcut.
  */
-bool createShortcut(QString destination, QString target, QStringList args, QString name, QString icon);
+QString createShortcut(QString destination, QString target, QStringList args, QString name, QString icon);
 
 enum class FilesystemType {
     FAT,
@@ -488,9 +494,9 @@ class clone : public QObject {
         m_src.setPath(src);
         m_dst.setPath(dst);
     }
-    clone& matcher(const IPathMatcher* filter)
+    clone& matcher(Filter filter)
     {
-        m_matcher = filter;
+        m_matcher = std::move(filter);
         return *this;
     }
     clone& whitelist(bool whitelist)
@@ -514,7 +520,7 @@ class clone : public QObject {
     bool operator()(const QString& offset, bool dryRun = false);
 
    private:
-    const IPathMatcher* m_matcher = nullptr;
+    Filter m_matcher = nullptr;
     bool m_whitelist = false;
     QDir m_src;
     QDir m_dst;
