@@ -114,9 +114,9 @@ VersionFilePtr OneSixVersionFormat::versionFileFromJson(const QJsonDocument& doc
         out->uid = root.value("fileId").toString();
     }
 
-    const QRegularExpression valid_uid_regex{ QRegularExpression::anchoredPattern(
+    static const QRegularExpression s_validUidRegex{ QRegularExpression::anchoredPattern(
         QStringLiteral(R"([a-zA-Z0-9-_]+(?:\.[a-zA-Z0-9-_]+)*)")) };
-    if (!valid_uid_regex.match(out->uid).hasMatch()) {
+    if (!s_validUidRegex.match(out->uid).hasMatch()) {
         qCritical() << "The component's 'uid' contains illegal characters! UID:" << out->uid;
         out->addProblem(ProblemSeverity::Error,
                         QObject::tr("The component's 'uid' contains illegal characters! This can cause security issues."));
@@ -176,7 +176,7 @@ VersionFilePtr OneSixVersionFormat::versionFileFromJson(const QJsonDocument& doc
         }
     }
 
-    auto readLibs = [&](const char* which, QList<LibraryPtr>& outList) {
+    auto readLibs = [&root, &out, &filename](const char* which, QList<LibraryPtr>& outList) {
         for (auto libVal : requireArray(root.value(which))) {
             QJsonObject libObj = requireObject(libVal);
             // parse the library
@@ -259,8 +259,8 @@ VersionFilePtr OneSixVersionFormat::versionFileFromJson(const QJsonDocument& doc
 
     if (root.contains("runtimes")) {
         out->runtimes = {};
-        for (auto runtime : ensureArray(root, "runtimes")) {
-            out->runtimes.append(Java::parseJavaMeta(ensureObject(runtime)));
+        for (auto runtime : root["runtimes"].toArray()) {
+            out->runtimes.append(Java::parseJavaMeta(runtime.toObject()));
         }
     }
 
