@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-3.0-only
 /*
- *  PolyMC - Minecraft Launcher
+ *  Prism Launcher - Minecraft Launcher
  *  Copyright (c) 2022 flowln <flowlnlnln@gmail.com>
  *  Copyright (C) 2022 Sefa Eyeoglu <contact@scrumplex.net>
  *
@@ -37,18 +37,18 @@
 #include "TranslationsModel.h"
 
 #include <QCoreApplication>
-#include <QTranslator>
-#include <QLocale>
+#include <QDebug>
 #include <QDir>
 #include <QLibraryInfo>
-#include <QDebug>
+#include <QLocale>
+#include <QTranslator>
 #include <locale>
 
-#include "FileSystem.h"
-#include "net/NetJob.h"
-#include "net/ChecksumValidator.h"
 #include "BuildConfig.h"
+#include "FileSystem.h"
 #include "Json.h"
+#include "net/ChecksumValidator.h"
+#include "net/NetJob.h"
 
 #include "POTranslator.h"
 
@@ -56,50 +56,35 @@
 
 const static QLatin1String defaultLangCode("en_US");
 
-enum class FileType
-{
-    NONE,
-    QM,
-    PO
-};
+enum class FileType { NONE, QM, PO };
 
-struct Language
-{
-    Language()
-    {
-        updated = true;
-    }
-    Language(const QString & _key)
+struct Language {
+    Language() { updated = true; }
+    Language(const QString& _key)
     {
         key = _key;
         locale = QLocale(key);
         updated = (key == defaultLangCode);
     }
 
-    QString languageName() const {
+    QString languageName() const
+    {
         QString result;
-        if(key == "ja_KANJI") {
+        if (key == "ja_KANJI") {
             result = locale.nativeLanguageName() + u8" (漢字)";
-        }
-        else if(key == "es_UY") {
-            result = u8"español de Latinoamérica";
-        }
-        else if(key == "en_NZ") {
-            result = u8"New Zealand English"; // No idea why qt translates this to just english and not to New Zealand English
-        }
-        else if(key == "en@pirate") {
+        } else if (key == "es_UY") {
+            result = u8"Español de Latinoamérica";
+        } else if (key == "en_NZ") {
+            result = u8"New Zealand English";  // No idea why qt translates this to just english and not to New Zealand English
+        } else if (key == "en@pirate") {
             result = u8"Tongue of the High Seas";
-        }
-        else if(key == "en@uwu") {
+        } else if (key == "en@uwu") {
             result = u8"Cute Engwish";
-        }
-        else if(key == "tok") {
+        } else if (key == "tok") {
             result = u8"toki pona";
-        }
-        else if(key == "nan") {
+        } else if (key == "nan") {
             result = u8"閩南語";  // Using traditional Chinese script. Not sure if we should use simplified instead?
-        }
-        else {
+        } else {
             result = locale.nativeLanguageName();
         }
 
@@ -111,8 +96,7 @@ struct Language
 
     float percentTranslated() const
     {
-        if (total == 0)
-        {
+        if (total == 0) {
             return 100.0f;
         }
         return 100.0f * float(translated) / float(total);
@@ -126,30 +110,17 @@ struct Language
         total = translated + untranslated + fuzzy;
     }
 
-    bool isOfSameNameAs(const Language& other) const
-    {
-        return key == other.key;
-    }
+    bool isOfSameNameAs(const Language& other) const { return key == other.key; }
 
     bool isIdenticalTo(const Language& other) const
     {
-        return
-        (
-            key == other.key &&
-            file_name == other.file_name &&
-            file_size == other.file_size &&
-            file_sha1 == other.file_sha1 &&
-            translated == other.translated &&
-            fuzzy == other.fuzzy &&
-            total == other.fuzzy &&
-            localFileType == other.localFileType
-        );
+        return (key == other.key && file_name == other.file_name && file_size == other.file_size && file_sha1 == other.file_sha1 &&
+                translated == other.translated && fuzzy == other.fuzzy && total == other.fuzzy && localFileType == other.localFileType);
     }
 
-    Language & apply(Language & other)
+    Language& apply(Language& other)
     {
-        if(!isOfSameNameAs(other))
-        {
+        if (!isOfSameNameAs(other)) {
             return *this;
         }
         file_name = other.file_name;
@@ -178,14 +149,11 @@ struct Language
     FileType localFileType = FileType::NONE;
 };
 
-
-
-struct TranslationsModel::Private
-{
+struct TranslationsModel::Private {
     QDir m_dir;
 
     // initial state is just english
-    QVector<Language> m_languages = {Language (defaultLangCode)};
+    QList<Language> m_languages = { Language(defaultLangCode) };
 
     QString m_selectedLanguage = defaultLangCode;
     std::unique_ptr<QTranslator> m_qt_translator;
@@ -198,7 +166,7 @@ struct TranslationsModel::Private
     QString m_nextDownload;
 
     std::unique_ptr<POTranslator> m_po_translator;
-    QFileSystemWatcher *watcher;
+    QFileSystemWatcher* watcher;
 
     const QString m_system_locale = QLocale::system().name();
     const QString m_system_language = m_system_locale.split('_').front();
@@ -206,7 +174,7 @@ struct TranslationsModel::Private
     bool no_language_set = false;
 };
 
-TranslationsModel::TranslationsModel(QString path, QObject* parent): QAbstractListModel(parent)
+TranslationsModel::TranslationsModel(QString path, QObject* parent) : QAbstractListModel(parent)
 {
     d.reset(new Private);
     d->m_dir.setPath(path);
@@ -218,15 +186,12 @@ TranslationsModel::TranslationsModel(QString path, QObject* parent): QAbstractLi
     d->watcher->addPath(d->m_dir.canonicalPath());
 }
 
-TranslationsModel::~TranslationsModel()
-{
-}
+TranslationsModel::~TranslationsModel() {}
 
 void TranslationsModel::translationDirChanged(const QString& path)
 {
     qDebug() << "Dir changed:" << path;
-    if (!d->no_language_set)
-    {
+    if (!d->no_language_set) {
         reloadLocalFiles();
     }
     selectLanguage(selectedLanguage());
@@ -237,126 +202,97 @@ void TranslationsModel::indexReceived()
     qDebug() << "Got translations index!";
     d->m_index_job.reset();
 
-    if (d->no_language_set)
-    {
+    if (d->no_language_set) {
         reloadLocalFiles();
 
         auto language = d->m_system_locale;
-        if (!findLanguageAsOptional(language).has_value())
-        {
+        if (!findLanguageAsOptional(language).has_value()) {
             language = d->m_system_language;
         }
         selectLanguage(language);
-        if (selectedLanguage() != defaultLangCode)
-        {
+        if (selectedLanguage() != defaultLangCode) {
             updateLanguage(selectedLanguage());
         }
         APPLICATION->settings()->set("Language", selectedLanguage());
         d->no_language_set = false;
     }
 
-    else if(d->m_selectedLanguage != defaultLangCode)
-    {
+    else if (d->m_selectedLanguage != defaultLangCode) {
         downloadTranslation(d->m_selectedLanguage);
     }
 }
 
 namespace {
-void readIndex(const QString & path, QMap<QString, Language>& languages)
+void readIndex(const QString& path, QMap<QString, Language>& languages)
 {
     QByteArray data;
-    try
-    {
+    try {
         data = FS::read(path);
-    }
-    catch (const Exception &e)
-    {
+    } catch ([[maybe_unused]] const Exception& e) {
         qCritical() << "Translations Download Failed: index file not readable";
         return;
     }
 
-    int index = 1;
-    try
-    {
+    try {
         auto toplevel_doc = Json::requireDocument(data);
         auto doc = Json::requireObject(toplevel_doc);
         auto file_type = Json::requireString(doc, "file_type");
-        if(file_type != "MMC-TRANSLATION-INDEX")
-        {
+        if (file_type != "MMC-TRANSLATION-INDEX") {
             qCritical() << "Translations Download Failed: index file is of unknown file type" << file_type;
             return;
         }
         auto version = Json::requireInteger(doc, "version");
-        if(version > 2)
-        {
+        if (version > 2) {
             qCritical() << "Translations Download Failed: index file is of unknown format version" << file_type;
             return;
         }
         auto langObjs = Json::requireObject(doc, "languages");
-        for(auto iter = langObjs.begin(); iter != langObjs.end(); iter++)
-        {
+        for (auto iter = langObjs.begin(); iter != langObjs.end(); iter++) {
             Language lang(iter.key());
 
-            auto langObj =  Json::requireObject(iter.value());
-            lang.setTranslationStats(
-                Json::ensureInteger(langObj, "translated", 0),
-                Json::ensureInteger(langObj, "untranslated", 0),
-                Json::ensureInteger(langObj, "fuzzy", 0)
-            );
+            auto langObj = Json::requireObject(iter.value());
+            lang.setTranslationStats(langObj["translated"].toInt(), langObj["untranslated"].toInt(),
+                                     langObj["fuzzy"].toInt());
             lang.file_name = Json::requireString(langObj, "file");
             lang.file_sha1 = Json::requireString(langObj, "sha1");
             lang.file_size = Json::requireInteger(langObj, "size");
 
             languages.insert(lang.key, lang);
-            index++;
         }
-    }
-    catch (Json::JsonException & e)
-    {
+    } catch ([[maybe_unused]] Json::JsonException& e) {
         qCritical() << "Translations Download Failed: index file could not be parsed as json";
     }
 }
-}
+}  // namespace
 
 void TranslationsModel::reloadLocalFiles()
 {
-    QMap<QString, Language> languages = {{defaultLangCode, Language(defaultLangCode)}};
+    QMap<QString, Language> languages = { { defaultLangCode, Language(defaultLangCode) } };
 
     readIndex(d->m_dir.absoluteFilePath("index_v2.json"), languages);
-    auto entries = d->m_dir.entryInfoList({"mmc_*.qm", "*.po"}, QDir::Files | QDir::NoDotAndDotDot);
-    for(auto & entry: entries)
-    {
+    auto entries = d->m_dir.entryInfoList({ "mmc_*.qm", "*.po" }, QDir::Files | QDir::NoDotAndDotDot);
+    for (auto& entry : entries) {
         auto completeSuffix = entry.completeSuffix();
         QString langCode;
         FileType fileType = FileType::NONE;
-        if(completeSuffix == "qm")
-        {
-            langCode = entry.baseName().remove(0,4);
+        if (completeSuffix == "qm") {
+            langCode = entry.baseName().remove(0, 4);
             fileType = FileType::QM;
-        }
-        else if(completeSuffix == "po")
-        {
+        } else if (completeSuffix == "po") {
             langCode = entry.baseName();
             fileType = FileType::PO;
-        }
-        else
-        {
+        } else {
             continue;
         }
 
         auto langIter = languages.find(langCode);
-        if(langIter != languages.end())
-        {
-            auto & language = *langIter;
-            if(int(fileType) > int(language.localFileType))
-            {
+        if (langIter != languages.end()) {
+            auto& language = *langIter;
+            if (int(fileType) > int(language.localFileType)) {
                 language.localFileType = fileType;
             }
-        }
-        else
-        {
-            if(fileType == FileType::PO)
-            {
+        } else {
+            if (fileType == FileType::PO) {
                 Language localFound(langCode);
                 localFound.localFileType = FileType::PO;
                 languages.insert(langCode, localFound);
@@ -365,52 +301,40 @@ void TranslationsModel::reloadLocalFiles()
     }
 
     // changed and removed languages
-    for(auto iter = d->m_languages.begin(); iter != d->m_languages.end();)
-    {
-        auto &language = *iter;
+    for (auto iter = d->m_languages.begin(); iter != d->m_languages.end();) {
+        auto& language = *iter;
         auto row = iter - d->m_languages.begin();
 
         auto updatedLanguageIter = languages.find(language.key);
-        if(updatedLanguageIter != languages.end())
-        {
-            if(language.isIdenticalTo(*updatedLanguageIter))
-            {
+        if (updatedLanguageIter != languages.end()) {
+            if (language.isIdenticalTo(*updatedLanguageIter)) {
                 languages.remove(language.key);
-            }
-            else
-            {
+            } else {
                 language.apply(*updatedLanguageIter);
                 emit dataChanged(index(row), index(row));
                 languages.remove(language.key);
             }
             iter++;
-        }
-        else
-        {
+        } else {
             beginRemoveRows(QModelIndex(), row, row);
             iter = d->m_languages.erase(iter);
             endRemoveRows();
         }
     }
     // added languages
-    if(languages.isEmpty())
-    {
+    if (languages.isEmpty()) {
         return;
     }
     beginInsertRows(QModelIndex(), 0, d->m_languages.size() + languages.size() - 1);
-    for(auto & language: languages)
-    {
+    for (auto& language : languages) {
         d->m_languages.append(language);
     }
     std::sort(d->m_languages.begin(), d->m_languages.end(), [this](const Language& a, const Language& b) {
-        if (a.key != b.key)
-        {
-            if (a.key == d->m_system_locale || a.key == d->m_system_language)
-            {
+        if (a.key != b.key) {
+            if (a.key == d->m_system_locale || a.key == d->m_system_language) {
                 return true;
             }
-            if (b.key == d->m_system_locale || b.key == d->m_system_language)
-            {
+            if (b.key == d->m_system_locale || b.key == d->m_system_language) {
                 return false;
             }
         }
@@ -420,14 +344,9 @@ void TranslationsModel::reloadLocalFiles()
 }
 
 namespace {
-enum class Column
-{
-    Language,
-    Completeness
-};
+enum class Column { Language, Completeness };
 }
 
-
 QVariant TranslationsModel::data(const QModelIndex& index, int role) const
 {
     if (!index.isValid())
@@ -439,62 +358,48 @@ QVariant TranslationsModel::data(const QModelIndex& index, int role) const
     if (row < 0 || row >= d->m_languages.size())
         return QVariant();
 
-    auto & lang = d->m_languages[row];
-    switch (role)
-    {
-    case Qt::DisplayRole:
-    {
-        switch(column)
-        {
-            case Column::Language:
-            {
-                return lang.languageName();
-            }
-            case Column::Completeness:
-            {
-                return QString("%1%").arg(lang.percentTranslated(), 3, 'f', 1);
+    auto& lang = d->m_languages[row];
+    switch (role) {
+        case Qt::DisplayRole: {
+            switch (column) {
+                case Column::Language: {
+                    return lang.languageName();
+                }
+                case Column::Completeness: {
+                    return QString("%1%").arg(lang.percentTranslated(), 3, 'f', 1);
+                }
             }
+            qWarning("TranslationModel::data not implemented when role is DisplayRole");
         }
-        qWarning("TranslationModel::data not implemented when role is DisplayRole");
-    }
-    case Qt::ToolTipRole:
-    {
-        return tr("%1:\n%2 translated\n%3 fuzzy\n%4 total").arg(lang.key, QString::number(lang.translated), QString::number(lang.fuzzy), QString::number(lang.total));
-    }
-    case Qt::UserRole:
-        return lang.key;
-    default:
-        return QVariant();
+        case Qt::ToolTipRole: {
+            return tr("%1:\n%2 translated\n%3 fuzzy\n%4 total")
+                .arg(lang.key, QString::number(lang.translated), QString::number(lang.fuzzy), QString::number(lang.total));
+        }
+        case Qt::UserRole:
+            return lang.key;
+        default:
+            return QVariant();
     }
 }
 
 QVariant TranslationsModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
     auto column = static_cast<Column>(section);
-    if(role == Qt::DisplayRole)
-    {
-        switch(column)
-        {
-            case Column::Language:
-            {
+    if (role == Qt::DisplayRole) {
+        switch (column) {
+            case Column::Language: {
                 return tr("Language");
             }
-            case Column::Completeness:
-            {
+            case Column::Completeness: {
                 return tr("Completeness");
             }
         }
-    }
-    else if(role == Qt::ToolTipRole)
-    {
-        switch(column)
-        {
-            case Column::Language:
-            {
+    } else if (role == Qt::ToolTipRole) {
+        switch (column) {
+            case Column::Language: {
                 return tr("The native language name.");
             }
-            case Column::Completeness:
-            {
+            case Column::Completeness: {
                 return tr("Completeness is the percentage of fully translated strings, not counting automatically guessed ones.");
             }
         }
@@ -502,19 +407,19 @@ QVariant TranslationsModel::headerData(int section, Qt::Orientation orientation,
     return QAbstractListModel::headerData(section, orientation, role);
 }
 
-int TranslationsModel::rowCount(const QModelIndex& parent) const
+int TranslationsModel::rowCount([[maybe_unused]] const QModelIndex& parent) const
 {
     return d->m_languages.size();
 }
 
-int TranslationsModel::columnCount(const QModelIndex& parent) const
+int TranslationsModel::columnCount([[maybe_unused]] const QModelIndex& parent) const
 {
     return 2;
 }
 
-QVector<Language>::Iterator TranslationsModel::findLanguage(const QString& key)
+QList<Language>::Iterator TranslationsModel::findLanguage(const QString& key)
 {
-    return std::find_if(d->m_languages.begin(), d->m_languages.end(), [&](Language& lang) { return lang.key == key; });
+    return std::find_if(d->m_languages.begin(), d->m_languages.end(), [key](Language& lang) { return lang.key == key; });
 }
 
 std::optional<Language> TranslationsModel::findLanguageAsOptional(const QString& key)
@@ -565,10 +470,8 @@ bool TranslationsModel::selectLanguage(QString key)
     QLocale::setDefault(
         QLocale(APPLICATION->settings()->get("UseSystemLocale").toBool() ? QString::fromStdString(std::locale().name()) : langCode));
 
-
     // if it's the default UI language, finish
-    if(langCode == defaultLangCode)
-    {
+    if (langCode == defaultLangCode) {
         d->m_selectedLanguage = langCode;
         return true;
     }
@@ -577,70 +480,47 @@ bool TranslationsModel::selectLanguage(QString key)
     bool successful = false;
     // FIXME: this is likely never present. FIX IT.
     d->m_qt_translator.reset(new QTranslator());
-    if (d->m_qt_translator->load("qt_" + langCode, QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
-    {
+    if (d->m_qt_translator->load("qt_" + langCode, QLibraryInfo::path(QLibraryInfo::TranslationsPath))) {
         qDebug() << "Loading Qt Language File for" << langCode.toLocal8Bit().constData() << "...";
-        if (!QCoreApplication::installTranslator(d->m_qt_translator.get()))
-        {
+        if (!QCoreApplication::installTranslator(d->m_qt_translator.get())) {
             qCritical() << "Loading Qt Language File failed.";
             d->m_qt_translator.reset();
-        }
-        else
-        {
+        } else {
             successful = true;
         }
-    }
-    else
-    {
+    } else {
         d->m_qt_translator.reset();
     }
 
-    if(langPtr->localFileType == FileType::PO)
-    {
+    if (langPtr->localFileType == FileType::PO) {
         qDebug() << "Loading Application Language File for" << langCode.toLocal8Bit().constData() << "...";
         auto poTranslator = new POTranslator(FS::PathCombine(d->m_dir.path(), langCode + ".po"));
-        if(!poTranslator->isEmpty())
-        {
-            if (!QCoreApplication::installTranslator(poTranslator))
-            {
+        if (!poTranslator->isEmpty()) {
+            if (!QCoreApplication::installTranslator(poTranslator)) {
                 delete poTranslator;
                 qCritical() << "Installing Application Language File failed.";
-            }
-            else
-            {
+            } else {
                 d->m_app_translator.reset(poTranslator);
                 successful = true;
             }
-        }
-        else
-        {
+        } else {
             qCritical() << "Loading Application Language File failed.";
             d->m_app_translator.reset();
         }
-    }
-    else if(langPtr->localFileType == FileType::QM)
-    {
+    } else if (langPtr->localFileType == FileType::QM) {
         d->m_app_translator.reset(new QTranslator());
-        if (d->m_app_translator->load("mmc_" + langCode, d->m_dir.path()))
-        {
+        if (d->m_app_translator->load("mmc_" + langCode, d->m_dir.path())) {
             qDebug() << "Loading Application Language File for" << langCode.toLocal8Bit().constData() << "...";
-            if (!QCoreApplication::installTranslator(d->m_app_translator.get()))
-            {
+            if (!QCoreApplication::installTranslator(d->m_app_translator.get())) {
                 qCritical() << "Installing Application Language File failed.";
                 d->m_app_translator.reset();
-            }
-            else
-            {
+            } else {
                 successful = true;
             }
-        }
-        else
-        {
+        } else {
             d->m_app_translator.reset();
         }
-    }
-    else
-    {
+    } else {
         d->m_app_translator.reset();
     }
     d->m_selectedLanguage = langCode;
@@ -650,8 +530,7 @@ bool TranslationsModel::selectLanguage(QString key)
 QModelIndex TranslationsModel::selectedIndex()
 {
     auto found = findLanguage(d->m_selectedLanguage);
-    if(found != d->m_languages.end())
-    {
+    if (found != d->m_languages.end()) {
         return index(std::distance(d->m_languages.begin(), found), 0, QModelIndex());
     }
     return QModelIndex();
@@ -664,17 +543,17 @@ QString TranslationsModel::selectedLanguage()
 
 void TranslationsModel::downloadIndex()
 {
-    if(d->m_index_job || d->m_dl_job)
-    {
+    if (d->m_index_job || d->m_dl_job) {
         return;
     }
     qDebug() << "Downloading Translations Index...";
     d->m_index_job.reset(new NetJob("Translations Index", APPLICATION->network()));
     MetaEntryPtr entry = APPLICATION->metacache()->resolveEntry("translations", "index_v2.json");
     entry->setStale(true);
-    auto task = Net::Download::makeCached(QUrl(BuildConfig.TRANSLATIONS_BASE_URL + "index_v2.json"), entry);
+    auto task = Net::Download::makeCached(QUrl(BuildConfig.TRANSLATION_FILES_URL + "index_v2.json"), entry);
     d->m_index_task = task.get();
     d->m_index_job->addNetAction(task);
+    d->m_index_job->setAskRetry(false);
     connect(d->m_index_job.get(), &NetJob::failed, this, &TranslationsModel::indexFailed);
     connect(d->m_index_job.get(), &NetJob::succeeded, this, &TranslationsModel::indexReceived);
     d->m_index_job->start();
@@ -682,33 +561,28 @@ void TranslationsModel::downloadIndex()
 
 void TranslationsModel::updateLanguage(QString key)
 {
-    if(key == defaultLangCode)
-    {
+    if (key == defaultLangCode) {
         qWarning() << "Cannot update builtin language" << key;
         return;
     }
     auto found = findLanguageAsOptional(key);
-    if(!found.has_value())
-    {
+    if (!found.has_value()) {
         qWarning() << "Cannot update invalid language" << key;
         return;
     }
-    if(!found->updated)
-    {
+    if (!found->updated) {
         downloadTranslation(key);
     }
 }
 
 void TranslationsModel::downloadTranslation(QString key)
 {
-    if(d->m_dl_job)
-    {
+    if (d->m_dl_job) {
         d->m_nextDownload = key;
         return;
     }
     auto lang = findLanguageAsOptional(key);
-    if(!lang.has_value())
-    {
+    if (!lang.has_value()) {
         qWarning() << "Will not download an unknown translation" << key;
         return;
     }
@@ -717,13 +591,13 @@ void TranslationsModel::downloadTranslation(QString key)
     MetaEntryPtr entry = APPLICATION->metacache()->resolveEntry("translations", "mmc_" + key + ".qm");
     entry->setStale(true);
 
-    auto dl = Net::Download::makeCached(QUrl(BuildConfig.TRANSLATIONS_BASE_URL + lang->file_name), entry);
-    auto rawHash = QByteArray::fromHex(lang->file_sha1.toLatin1());
-    dl->addValidator(new Net::ChecksumValidator(QCryptographicHash::Sha1, rawHash));
+    auto dl = Net::Download::makeCached(QUrl(BuildConfig.TRANSLATION_FILES_URL + lang->file_name), entry);
+    dl->addValidator(new Net::ChecksumValidator(QCryptographicHash::Sha1, lang->file_sha1));
     dl->setProgress(dl->getProgress(), lang->file_size);
 
     d->m_dl_job.reset(new NetJob("Translation for " + key, APPLICATION->network()));
     d->m_dl_job->addNetAction(dl);
+    d->m_dl_job->setAskRetry(false);
 
     connect(d->m_dl_job.get(), &NetJob::succeeded, this, &TranslationsModel::dlGood);
     connect(d->m_dl_job.get(), &NetJob::failed, this, &TranslationsModel::dlFailed);
@@ -733,8 +607,7 @@ void TranslationsModel::downloadTranslation(QString key)
 
 void TranslationsModel::downloadNext()
 {
-    if(!d->m_nextDownload.isEmpty())
-    {
+    if (!d->m_nextDownload.isEmpty()) {
         downloadTranslation(d->m_nextDownload);
         d->m_nextDownload.clear();
     }
@@ -751,8 +624,7 @@ void TranslationsModel::dlGood()
 {
     qDebug() << "Got translation:" << d->m_downloadingTranslation;
 
-    if(d->m_downloadingTranslation == d->m_selectedLanguage)
-    {
+    if (d->m_downloadingTranslation == d->m_selectedLanguage) {
         selectLanguage(d->m_selectedLanguage);
     }
     d->m_dl_job.reset();
